# PASDy.Config

import os
import re
from configobj import ConfigObj # better alternative to configparser: preserves order and comments
from copy import deepcopy

import PASDy.Files as Files
import PASDy.Misc as Misc
from PASDy.Misc import printv

#
#
class Config:
   
    # config = PASDy.Config(file)
    # creates a Config object from file.cfg

    def __init__(self, file, default_file='config.cfg', encoding='cp850', verbose=2,
                 reprocess=True, clean_paths=True, convert_bool=True, unfold_range=True, join_lists=True,
                 unfold_numunits=[]):
        self.file = file
        self.default_file = default_file
        self.encoding = encoding
        self._verbose = verbose
        self.reprocess = reprocess
        self.clean_paths = clean_paths
        self.convert_bool = convert_bool
        self.unfold_range = unfold_range
        self.join_lists = join_lists
        self.unfold_numunits = unfold_numunits
        
        self.c = None
        self.read()
    
    # Verbose print
    def _print(self, level, s=''):
        printv(level, self._verbose, s)        
    
    # Read
    def read(self):
        if self.file == '' or self.file is None or not self.file:
            # os.path.abspath('') is better than __file__ most of the time, see https://stackoverflow.com/questions/39125532/file-does-not-exisit-in-jupyter-notebook
            self.file = self.default_file
            self._print(2, 'No file given, choosing default %s' % default_file)
        
        self.c = config_read(self.file, self.encoding, self._verbose)

        if not self.c is None:
            self.sections = self.c.sections
            if self.reprocess:
                self.do_process()

    # Process
    def do_process(self):
        self.cp = deepcopy(self.c)
        
        for section in self.sections:
            for k in self.c[section]:
                v = self.c[section][k]
                
                if 'path' in k:
                    if self.clean_paths:
                        self.cp[section][k] = Files.trailingslash(v)
                        
                elif isinstance(v, list):
                    if self.join_lists:
                        self.cp[section][k] = ', '.join(v)
                
                elif v.lower() == 'yes' or v.lower() == 'no':
                    if self.convert_bool:                
                        self.cp[section][k] = Misc.yesno2bool(v)
                        
                elif k.endswith('_range'):
                    if self.unfold_range:
                        if not isinstance(v, list):
                            v = ['','']
                        self.cp[section][k[:k.index('_range')]+'_min'] = v[0]
                        self.cp[section][k[:k.index('_range')]+'_max'] = v[1]
                    
                elif self.unfold_numunits:
                    if k in self.unfold_numunits:
                        if v:
                            v_bundle = re.search('^(\d+)([a-z]+)', v, re.IGNORECASE)
                            self.cp[section][k] = [v_bundle.group(1), v_bundle.group(2)]
                        else:
                            self.cp[section][k] = ['','']

                        self.cp[section][k+'_num'] = v[0]
                        self.cp[section][k+'_unit'] = v[1]
    
    def undo_process(self):
        
        for section in self.sections:
            for k in self.cp[section]:
                v = self.cp[section][k]
                
                if isinstance(v, bool):
                    if self.convert_bool:
                        self.c[section][k] = Misc.bool2yesno(v)
                    
                elif ',' in v:
                    if self.join_lists:
                        self.c[section][k] = [x.strip() for x in v.split(',')]
                
                elif k.endswith('_range'):
                    if self.unfold_range:
                        kcut = k[:k.index('_range')]
                        self.c[section][k] = [ self.cp[section][kcut+'_min'], self.cp[section][kcut+'_max'] ]
                        if self.cp[section][kcut+'_min'] == '':
                            self.c[section][k] = ''
                        
                elif self.unfold_numunits:
                    if k in self.unfold_numunits:
                        if self.cp[section][k+'_num'] == '':
                            self.c[section][k] = ''
                        else:
                            self.c[section][k] = self.cp[section][k+'_num'] + self.cp[section][k+'_unit']
    
    # Save
    def save(self, file=None):
        self.undo_process()
        # no argument given
        if file is None: file = self.file
        # Save
        config_save(self.c, file, self._verbose)
        

# Functions
#
## Read ConfigObj from file.cfg
##
def config_read(file, encoding='cp850', verbose=1):
    
    if os.path.isfile(file):
        printv(1, verbose, 'Reading sensor config: %s' % file)
        return(ConfigObj(file, encoding=encoding))
    else:
        print(1, verbose, 'File does not exist: %s' % file)
        return(None)


## Save ConfigObj into file
##
def config_save(config, file, verbose=1):
    
    if file == '' or file is None or not file:
        # invalid file
        printv(0, verbose, 'Cannot save config file: %s' % file)
        return(False)
    else:
        config.filename = file
        config.write_empty_values = True
        config.write()
        # TODO: check if write() was successful
        Files.file_save_msg(file, name='Config')
        return(True)